<?php

/*
Plugin Name: Elegance Countries
Description: Import and manage a list of countries into your WordPress site as Custom Post Types.
Version: 1.0.0
Author: Elegance Team
Author URI: https://elegancedesign.net/
License: GPL2
*/

class Countries {

	public $post_type = 'country';
	public $has_countries = false;
	
	/**
	 * Constructor
	 */
	function __construct() {
		add_action( 'init', array( $this, 'register_post_types' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_script' ) );

		if ( is_admin() ) {
			$this->has_countries = !get_posts(['post_type' => $this->post_type]);
			add_action( 'admin_menu', array( $this, 'admin_menu' ) );
			add_action( 'current_screen', array( $this, 'check_import_screen' ) );
			add_action( 'save_post', array( $this, 'save_details' ) );
			add_filter( 'posts_orderby', array( $this, 'countries_orderby' ), 0, 1 );
			add_action( 'wp_ajax_start_importing', array( $this, 'start_importing' ) );
			add_action( 'wp_ajax_nopriv_start_importing', array( $this, 'start_importing') );
		}
	}

	/**
	 * Create custom post type and custom taxonomy
	 * http://codex.wordpress.org/Function_Reference/register_post_type
	 */
	function register_post_types() {
		$labels = array(
			'name'               => _x( 'Countries', 'post type general name', 'countries' ),
			'all_items'          => __( 'All Countries', 'countries'),
			'singular_name'      => _x( 'Country', 'post type singular name', 'countries' ),
			'add_new'            => _x( 'Add New', 'Country', 'countries' ),
			'add_new_item'       => __( 'Add New Country', 'countries' ),
			'edit_item'          => __( 'Edit Country', 'countries' ),
			'new_item'           => __( 'New Country', 'countries' ),
			'view_item'          => __( 'View Country', 'countries' ),
			'search_items'       => __( 'Search Countries', 'countries' ),
			'not_found'          => __( 'No Countries found', 'countries' ),
			'not_found_in_trash' => __( 'No Countries found in Trash', 'countries' ), 
			'menu_name'          => __( 'Countries', 'countries' )
		);
		$args = array(
			'labels'               => $labels,
			'public'               => false,
			'exclude_from_search'  => true,
			'publicly_queryable'   => true,
			'show_ui'              => true,
			'show_in_nav_menus'    => false, 
			'query_var'            => true,
			'rewrite'              => true,
			'capability_type'      => 'post',
			'hierarchical'         => false,
			'show_in_nav_menus'    => false,
			'menu_position'        => 58,
			'menu_icon'            => 'dashicons-admin-site',
			'supports'             => array( 'title', 'thumbnail'),
			'visible'              => true,
			'register_meta_box_cb' => array( $this, 'add_country_meta_boxes' )
		);
		register_post_type($this->post_type, $args );
	}

	/**
	 * Add Country Sripts and Styles
	 */
	function enqueue_script () {
		if ($this->has_countries) {
			wp_enqueue_script( $this->post_type, plugin_dir_url( __FILE__ ) . 'js/countries.js', ['jquery'], '1.0.0');
		}
	}

	/**
	 * Add Country Meta Boxes
	 */

	function add_country_meta_boxes() {
		add_meta_box('country_meta', __( 'Country Code', 'countries' ), array( $this, 'country_code_meta' ), $this->post_type, 'normal', 'low');
	}

	/**
	 * Render Country Code Meta Box
	 */
	function country_code_meta() {
		global $post;
		$country_code = get_post_meta( $post->ID, 'country_code', true );
		?>
		<input type="text" name="country_code" value="<?php echo $country_code; ?>" />
		<?php
	}
	
	/**
	 * Countries Admin Menu
	 *
	 * Adds XML Importer admin page.
	 */
	function admin_menu() {
		if ($this->has_countries) {
			add_submenu_page( 'edit.php?post_type='. $this->post_type, __( 'Countries Importer', 'countries' ), __( 'Import', 'countries' ), 'manage_options', 'import-countries', [ $this, 'importer_page' ] );
		}
	}

	function check_import_screen() {
		if ($this->has_countries) {
			$screen = get_current_screen();
			if ($screen->id !== 'country_page_import-countries') {
				wp_redirect('edit.php?post_type='. $this->post_type . '&page=import-countries');
				exit;
			}
		}
	}

	/**
	 * Countries Import Page
	 */
	function importer_page() {
		include_once( plugin_dir_path( __FILE__ ) . 'includes/countries.php' );
		if ($this->has_countries) {
	?>
			<div class="wrap">
				<h2><?php _e( 'Countries Importer', 'countries' ); ?></h2>
				<p><?php _e( 'Countries are being imported, please wait until the process is complete.', 'countries' ); ?></p>
				<p><strong><?php _e( 'Imported', 'countries' ); ?> <span class="country-imported-count">0</span> <?php _e( 'of', 'countries' ); echo ' ' . count($countries); ?></strong></p>
				<div class="country-import"></div>
				<p class="country-importing"><?php _e('Importing...', 'countries'); ?></p>
				<p class="country-import-last"><strong><?php _e( 'Imported', 'countries' ); ?> <span class="country-imported-count">0</span> <?php _e( 'of', 'countries' ); echo ' ' . count($countries); ?></strong></p>
			</div>
	<?php
		}
	}

	function start_importing() {
		$index = $_GET['index'];
		include_once( plugin_dir_path( __FILE__ ) . 'includes/countries.php' );
		
		$country = $countries[$index];
		$count = count($countries);
		
		$country_id = wp_insert_post([
			'post_title' => $country['name'],
			'post_name' => $country['code'],
			'post_status' => 'publish',
			'post_author' => 1,
			'post_type' => 'country'
		]);

		update_post_meta($country_id, 'country_code', $country['code']);

		$image_url = plugin_dir_path( __FILE__ ) . 'flags/' . mb_strtolower($country['code']) . '.svg';
		$this->generate_featured_image($image_url, $country_id);

		$html .= '<div>' . $country['name'] . ' (' . $country['code'] . ') - <em>' . __( 'Imported', 'countries' ) . '</em></div>';
		if ($index == $count - 1) {
			$html .= '<p><strong>' . sprintf( __( '%s countries imported.', 'countries' ), $count ) . '</strong> <a	href="'. get_dashboard_url() .'">'. __('Go to Dashboard', 'contruies') .'</a></p>';
		}
		
		echo json_encode([
			'html' => $html,
			'count' => $count
		]);
		wp_die();
	}


	/**
	 * Generate Country Flag
	 * 
	 * @param string $image_url Image URL
	 * @param int $post_id Post ID.
	 */
	function generate_featured_image($image_url, $post_id) {
    $upload_dir = wp_upload_dir();
		$image_data = file_get_contents($image_url);
    $filename = basename($image_url);
		$unique_file_name = wp_unique_filename( $upload_dir['path'], $filename);
		$filename = basename($unique_file_name);
    if(wp_mkdir_p($upload_dir['path']))
      $file = $upload_dir['path'] . '/' . $filename;
    else
      $file = $upload_dir['basedir'] . '/' . $filename;
    file_put_contents($file, $image_data);

    $wp_filetype = wp_check_filetype($filename, null );
    $attachment = array(
        'post_mime_type' => $wp_filetype['type'],
        'post_title' => sanitize_file_name($filename),
        'post_content' => '',
        'post_status' => 'inherit'
    );
    $attach_id = wp_insert_attachment( $attachment, $file, $post_id );
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    $attach_data = wp_generate_attachment_metadata( $attach_id, $file );
    wp_update_attachment_metadata( $attach_id, $attach_data );
    set_post_thumbnail( $post_id, $attach_id );
	}
	
	/**
	 * Save Country Meta Box Details
	 *
	 * @param int $post_id Post ID.
	 */
	function save_details( $post_id ) {
		
		$post_vars = shortcode_atts( array(
			'country_code' => ''
		), $_POST );
		
		update_post_meta( $post_id, 'country_code', $post_vars['country_code'] );
	}
	
	
	/**
	 * Countries orderby
	 *
	 * Forces countries to default to order alphabetically in the admin rather
	 * than by publish date. Does this as priority 0 so that by default any other
	 * filters will override this.
	 *
	 * @todo Should we do this by default elsewhere, not just the admin?
	 *
	 * @param string $order The SQL order statement.
	 */
	function countries_orderby( $order ) {
		global $wpdb;
		if ( get_query_var( 'post_type' ) == $this->post_type ) {
			return "$wpdb->posts.post_title ASC";
		}
		return $order;
	}
}

new Countries();

require_once(plugin_dir_path( __FILE__ ) . 'includes/functions.php');
